//
//  CommunicationProtocol.cpp
//  24U Phidgets Plug-In
//
//  Created by Patrik Horcicak on 23/09/2019.
//  Copyright © 2019 Filemaker, Inc. All rights reserved.
//

#include "CommunicationProtocol.h"
//#include "AppleBluetoothManager.h"
#include "SuppFunc.h"

CommunicationProtocol& CommunicationProtocol::sharedInstance()
{
    static CommunicationProtocol g_CommunicationProtocol{};
    return g_CommunicationProtocol;
}

CommunicationProtocol::CommunicationProtocol()
{
    
}

CommunicationProtocol::~CommunicationProtocol()
{
    
}

std::string CommunicationProtocol::prepareData(const PhidgetHandlers* device, PLUGIN_FUNC function, PLUGIN_FUNC_SELECTOR selector, std::vector<std::string>params)
{
    std::stringstream resultData;
    
    if (device == nullptr)
    {
        resultData
        << ""       << STR_SEPARATOR
        << ""       << STR_SEPARATOR
        << ""       << STR_SEPARATOR
        << ""       << STR_SEPARATOR
        << ""       << STR_SEPARATOR
        << ""       << STR_SEPARATOR
        << ""       << STR_SEPARATOR
        << ""       << STR_SEPARATOR
        << ""       << STR_SEPARATOR
        << ""       << STR_SEPARATOR
        << ""       << STR_SEPARATOR
        << ""       << STR_SEPARATOR
        << function << STR_SEPARATOR
        << selector << STR_SEPARATOR;
    }
    else
    {
        resultData
        <<  device->getProperty(PhidgetProperty::SERIAL_NUMBER)      << STR_SEPARATOR
        <<  device->getProperty(PhidgetProperty::HUB_PORT)           << STR_SEPARATOR
        <<  device->getProperty(PhidgetProperty::CHANNEL)            << STR_SEPARATOR
        <<  device->getProperty(PhidgetProperty::CHANNEL_CLASS)      << STR_SEPARATOR
        <<  device->getProperty(PhidgetProperty::CHANNEL_SUBCLASS)   << STR_SEPARATOR
        <<  device->getProperty(PhidgetProperty::DEVICE_CLASS_NAME)  << STR_SEPARATOR
        <<  device->getProperty(PhidgetProperty::DEVICE_NAME)        << STR_SEPARATOR
        <<  device->getProperty(PhidgetProperty::DEVICE_LABEL)       << STR_SEPARATOR
        <<  device->getProperty(PhidgetProperty::DEVICE_VERSION)     << STR_SEPARATOR
        <<  device->getProperty(PhidgetProperty::DEVICE_ID)          << STR_SEPARATOR
        <<  device->getProperty(PhidgetProperty::IS_HUB_PORT_DEVICE) << STR_SEPARATOR
        <<  device->getProperty(PhidgetProperty::IS_REMOTE)          << STR_SEPARATOR
        <<  function                                                 << STR_SEPARATOR
        <<  selector                                                 << STR_SEPARATOR;
    }
    
    for (size_t i = 0; i < params.size(); i++)
    {
        resultData << params[i];
        if (i != params.size() - 1)
        {
            resultData << STR_SEPARATOR;
        }
    }
    
    return resultData.str();
}

void CommunicationProtocol::messageReceived(std::string deviceName, std::vector<std::string> message)
{
	#ifdef _WIN32
	
	auto it = Manager::GetManager().connectedDevices.find(deviceName);
	if (it != Manager::GetManager().connectedDevices.end())
	{
		if (it->second->connectionType == BT)
		{	
			receive_mutex.lock();
			it->second->manager->response = message;
		}
	}

	#endif


    if (message.size() == 0)
    {
        return;
    }
    if (message.at(0) == "SENDING ATTACHED PHIDGETS")
    {
        std::vector<std::vector<std::string> > allPhidgetStates =  phidgetStatesFromVector(message);
        
        for (auto ps : allPhidgetStates)
        {
            messageReceived("localhost", ps);
        }
		#ifdef _WIN32
		if (it->second->connectionType == BT)
		{
			receive_mutex.unlock();
		}
		#endif
        return;
    }
    
    //Parse message
    if (message.size() > 13)
    {
        PLUGIN_FUNC function = PLUGIN_FUNC(std::stoi(message.at(12)));
        
        //Process Set Script Event
        if (function == PLUGIN_FUNC::PLUGIN_SCRIPT_EVENT)
        {
            PLUGIN_FUNC_SELECTOR selector = PLUGIN_FUNC_SELECTOR(std::stoi(message.at(13)));
            
            switch (selector)
            {
                case SE_ON_ATTACH:
                {
                    Manager::GetManager().AttachHandlerRemote(deviceName, message);
                    break;
                }
                case SE_ON_DETTACH:
                {
                    Manager::GetManager().DetachHandlerRemote(deviceName, message);
                    break;
                }
                case SE_ON_TAG:
                {
                    int serialNumber = std::stoi(message.at(0));
                    int hubPort      = std::stoi(message.at(1));
                    
                    std::string protocolConverted = message.at(14);
                    const char * tag = message.at(15).c_str();
                    
                    std::string device = std::to_string(serialNumber) + std::string{":"} + std::to_string(hubPort);

//                    std::string protocolConverted{ConvertToBackwardCompatible(std::string{tag})};
                    Manager::GetManager().EventHandler(device, "ontag", 0, std::vector<std::string>{protocolConverted, std::string{tag}}); // params for script index tagBackwardCompatibleString tagValueString
                    break;
                }
                case SE_ON_TAG_LOST:
                {
                    int serialNumber = std::stoi(message.at(0));
                    int hubPort      = std::stoi(message.at(1));
                    
                    std::string protocolConverted = message.at(14);
                    const char * tag = message.at(15).c_str();
                    
                    std::string device = std::to_string(serialNumber) + std::string{":"} + std::to_string(hubPort);

//                    std::string protocolConverted{ConvertToBackwardCompatible(std::string{tag})};
                    Manager::GetManager().EventHandler(device, "ontaglost", 0, std::vector<std::string>{protocolConverted, std::string{tag}}); // params for script index tagBackwardCompatibleString tagValueString
                    break;
                }
                case SE_ON_BRIDGE_DATA:
                {
                    int serialNumber = std::stoi(message.at(0));
                    int hubPort      = std::stoi(message.at(1));
                    int channel      = std::stoi(message.at(2));
                    
                    double voltageRatio = std::stod(message.at(14));
                    
                    std::string device = std::to_string(serialNumber) + std::string{":"} + std::to_string(hubPort);
                     
                    Manager::GetManager().EventHandler(device, "onbridgedata", channel, std::vector<std::string>{std::to_string(voltageRatio*1000) });
                    break;
                }
                case SE_ON_INPUT_CHANGE:
                {
                    int serialNumber = std::stoi(message.at(0));
                    int hubPort      = std::stoi(message.at(1));
                    int channel      = std::stoi(message.at(2));
                    
                    int state        = std::stoi(message.at(14));
                    
                    std::string device = std::to_string(serialNumber) + std::string{":"} + std::to_string(hubPort);;

                    Manager::GetManager().EventHandler(device, "oninputchange", channel, std::vector<std::string>{std::to_string(state)});
                    break;
                }
                case SE_ON_SENSOR_CHANGE:
                {
                    int serialNumber = std::stoi(message.at(0));
                    int hubPort      = std::stoi(message.at(1));
                    int channel      = std::stoi(message.at(2));
                    
                    double pH = std::stod(message.at(14));
                    
                    
                    std::string device = std::to_string(serialNumber) + std::string{":"} + std::to_string(hubPort);

                    Manager::GetManager().EventHandler(device, "onsensorchange", channel, std::vector<std::string>{std::to_string(pH)});
                    break;
                }
                case SE_ON_OUTPUT_CHANGE:
                {
                    int serialNumber = std::stoi(message.at(0));
                    int hubPort      = std::stoi(message.at(1));
                    int channel      = std::stoi(message.at(2));
                    
                    int state = std::stoi(message.at(14));
                    
                    std::string device = std::to_string(serialNumber) + std::string{":"} + std::to_string(hubPort);
                    
                    Manager::GetManager().EventHandler(device, "onoutputchange", channel, std::vector<std::string>{std::to_string(state)});
                    break;
                }
                case SE_MOTOR_POS_CHANGE:
                {
                    int serialNumber = std::stoi(message.at(0));
                    int hubPort      = std::stoi(message.at(1));
                    int channel      = std::stoi(message.at(2));
                    
                    int position = std::stoi(message.at(14));
                    
                    std::string device = std::to_string(serialNumber) + std::string{":"} + std::to_string(hubPort);

                    Manager::GetManager().EventHandler(device, "onmotorposchange", channel, std::vector<std::string>{std::to_string(position)});
                    break;
                }
                case SE_ON_STEPPER_STOP:
                {
                    int serialNumber = std::stoi(message.at(0));
                    int hubPort      = std::stoi(message.at(1));
                    int channel      = std::stoi(message.at(2));
                    
                    int position = std::stoi(message.at(14));
                    
                    std::string device = std::to_string(serialNumber) + std::string{":"} + std::to_string(hubPort);
                    
                    Manager::GetManager().EventHandler(device, "onstepperstop", channel, std::vector<std::string>{std::to_string(position)});
                    break;
                }
                    
                default: { break; }
            }
        }
    }
	#ifdef _WIN32
	if (it->second->connectionType == BT)
	{
		receive_mutex.unlock();
	}
	#endif
	return;
}

std::vector<std::vector<std::string > > CommunicationProtocol::phidgetStatesFromVector(std::vector<std::string> phidgetsVec)
{
    // phidgets.at(0) = "SENDING ATTACHED PHIDGETS"
    
    std::vector<std::vector<std::string> >  phidgetStates;
    std::vector<std::string>                ps;
    
    int i   = 1;
    
    while (i <= phidgetsVec.size() - 14)
    {
        for (int j = 0; j < 14; j++)
        {
            ps.push_back(phidgetsVec.at(i++));
        }
        
        phidgetStates.push_back(ps);
        ps.clear();
    }
    
    return phidgetStates;
}

